@@ -834,7 +834,7 @@ static int _read_io_init_msg(int fd, client_io_t *cio, slurm_addr_t *host)
 		error("failed reading io init message");
 		goto fail;
 	}
-	if (io_init_msg_validate(&msg, cio->io_key) < 0) {
+	if (io_init_msg_validate(&msg, cio->io_key, cio->io_key_len) < 0) {
 		goto fail;
 	}
 	if (msg.nodeid >= cio->num_nodes) {
