@@ -63,10 +63,6 @@
 #include "src/common/xassert.h"
 #include "src/slurmdbd/read_config.h"
 
-#define MAX_ARRAY_LEN_SMALL	10000
-#define MAX_ARRAY_LEN_MEDIUM	1000000
-#define MAX_ARRAY_LEN_LARGE	100000000
-
 /*
  * Define slurm-specific aliases for use by plugins, see slurm_xlator.h
  * for details.
@@ -529,8 +525,6 @@ int unpack16_array(uint16_t **valp, uint32_t *size_val, buf_t *buffer)
 
 	if (unpack32(size_val, buffer))
 		return SLURM_ERROR;
-	if ((*size_val) > MAX_ARRAY_LEN_MEDIUM)
-		return SLURM_ERROR;
 
 	*valp = xmalloc_nz((*size_val) * sizeof(uint16_t));
 	for (i = 0; i < *size_val; i++) {
@@ -565,8 +559,6 @@ int unpack32_array(uint32_t **valp, uint32_t *size_val, buf_t *buffer)
 
 	if (unpack32(size_val, buffer))
 		return SLURM_ERROR;
-	if ((*size_val) > MAX_ARRAY_LEN_LARGE)
-		return SLURM_ERROR;
 
 	*valp = xmalloc_nz((*size_val) * sizeof(uint32_t));
 	for (i = 0; i < *size_val; i++) {
@@ -600,8 +592,6 @@ int unpack64_array(uint64_t **valp, uint32_t *size_val, buf_t *buffer)
 
 	if (unpack32(size_val, buffer))
 		return SLURM_ERROR;
-	if ((*size_val) > MAX_ARRAY_LEN_MEDIUM)
-		return SLURM_ERROR;
 
 	*valp = xmalloc_nz((*size_val) * sizeof(uint64_t));
 	for (i = 0; i < *size_val; i++) {
@@ -630,8 +620,6 @@ int unpackdouble_array(double **valp, uint32_t* size_val, buf_t *buffer)
 
 	if (unpack32(size_val, buffer))
 		return SLURM_ERROR;
-	if ((*size_val) > MAX_ARRAY_LEN_SMALL)
-		return SLURM_ERROR;
 
 	*valp = xmalloc_nz((*size_val) * sizeof(double));
 	for (i = 0; i < *size_val; i++) {
@@ -661,8 +649,6 @@ int unpacklongdouble_array(long double **valp, uint32_t *size_val,
 
 	if (unpack32(size_val, buffer))
 		return SLURM_ERROR;
-	if ((*size_val) > MAX_ARRAY_LEN_SMALL)
-		return SLURM_ERROR;
 
 	*valp = xmalloc_nz((*size_val) * sizeof(long double));
 	for (i = 0; i < *size_val; i++) {
@@ -832,9 +818,9 @@ int unpackmem_ptr(char **valp, uint32_t *size_valp, buf_t *buffer)
 	*size_valp = ntohl(ns);
 	buffer->processed += sizeof(ns);
 
-	if (*size_valp > MAX_ARRAY_LEN_LARGE) {
+	if (*size_valp > MAX_PACK_MEM_LEN) {
 		error("%s: Buffer to be unpacked is too large (%u > %u)",
-		      __func__, *size_valp, MAX_ARRAY_LEN_LARGE);
+		      __func__, *size_valp, MAX_PACK_MEM_LEN);
 		return SLURM_ERROR;
 	}
 	else if (*size_valp > 0) {
@@ -867,9 +853,9 @@ int unpackmem_xmalloc(char **valp, uint32_t *size_valp, buf_t *buffer)
 	*size_valp = ntohl(ns);
 	buffer->processed += sizeof(ns);
 
-	if (*size_valp > MAX_ARRAY_LEN_LARGE) {
+	if (*size_valp > MAX_PACK_MEM_LEN) {
 		error("%s: Buffer to be unpacked is too large (%u > %u)",
-		      __func__, *size_valp, MAX_ARRAY_LEN_LARGE);
+		      __func__, *size_valp, MAX_PACK_MEM_LEN);
 		return SLURM_ERROR;
 	}
 	else if (*size_valp > 0) {
@@ -903,9 +889,10 @@ int unpackmem_malloc(char **valp, uint32_t *size_valp, buf_t *buffer)
 	memcpy(&ns, &buffer->head[buffer->processed], sizeof(ns));
 	*size_valp = ntohl(ns);
 	buffer->processed += sizeof(ns);
-	if (*size_valp > MAX_ARRAY_LEN_SMALL) {
+
+	if (*size_valp > MAX_PACK_MEM_LEN) {
 		error("%s: Buffer to be unpacked is too large (%u > %u)",
-		      __func__, *size_valp, MAX_ARRAY_LEN_SMALL);
+		      __func__, *size_valp, MAX_PACK_MEM_LEN);
 		return SLURM_ERROR;
 	}
 	else if (*size_valp > 0) {
@@ -1048,12 +1035,7 @@ int unpackstr_array(char ***valp, uint32_t *size_valp, buf_t *buffer)
 	*size_valp = ntohl(ns);
 	buffer->processed += sizeof(ns);
 
-	if (*size_valp > MAX_ARRAY_LEN_MEDIUM) {
-		error("%s: Buffer to be unpacked is too large (%u > %u)",
-		      __func__, *size_valp, MAX_ARRAY_LEN_MEDIUM);
-		return SLURM_ERROR;
-	}
-	else if (*size_valp > 0) {
+	if (*size_valp > 0) {
 		*valp = xmalloc_nz(sizeof(char *) * (*size_valp + 1));
 		for (i = 0; i < *size_valp; i++) {
 			if (unpackmem_xmalloc(&(*valp)[i], &uint32_tmp, buffer))
