@@ -178,7 +178,8 @@ int stepd_signal_container(int fd, uint16_t protocol_version, int signal,
  */
 int stepd_attach(int fd, uint16_t protocol_version,
 		 slurm_addr_t *ioaddr, slurm_addr_t *respaddr,
-		 void *job_cred_sig, reattach_tasks_response_msg_t *resp);
+		 void *job_cred_sig, uid_t uid,
+		 reattach_tasks_response_msg_t *resp);
 
 /*
  * Scan for available running slurm step daemons by checking
