@@ -158,7 +158,6 @@ static const char *NODE_DESC = "Node";
 static int dataset_id = -1; /* id of the dataset for profile data */
 
 static slurm_ipmi_conf_t slurm_ipmi_conf;
-static uint64_t debug_flags = 0;
 static bool flag_energy_accounting_shutdown = false;
 static bool flag_thread_started = false;
 static bool flag_init = false;
@@ -454,8 +453,8 @@ static int _find_power_sensor(void)
 
 	if (rc != SLURM_SUCCESS)
 		info("Power sensor not found.");
-	else if (debug_flags & DEBUG_FLAG_ENERGY)
-		info("Power sensor found: %d", sensors_len);
+	else
+		log_flag(ENERGY, "Power sensor found: %d", sensors_len);
 
 	return rc;
 }
@@ -578,7 +577,7 @@ static int _thread_update_node_energy(void)
 
 	readings++;
 
-	if (debug_flags & DEBUG_FLAG_ENERGY) {
+	if (slurm_conf.debug_flags & DEBUG_FLAG_ENERGY) {
 		for (i = 0; i < sensors_len; ++i)
 			info("ipmi-thread: sensor %u current_watts: %u, consumed %"PRIu64" Joules %"PRIu64" new, ave watts %u",
 			     sensors[i].id,
@@ -601,7 +600,7 @@ static int _thread_init(void)
 	int rc = SLURM_SUCCESS;
 	uint16_t i;
 
-	if (!first)
+	if (!first && ipmi_ctx)
 		return first_init;
 	first = false;
 
@@ -632,8 +631,7 @@ static int _thread_init(void)
 		if (ipmi_ctx)
 			ipmi_monitoring_ctx_destroy(ipmi_ctx);
 
-	if (debug_flags & DEBUG_FLAG_ENERGY)
-		info("%s thread init", plugin_name);
+	log_flag(ENERGY, "%s thread init", plugin_name);
 
 	first_init = SLURM_SUCCESS;
 
@@ -663,8 +661,7 @@ static int _ipmi_send_profile(void)
 			"Energy", NO_PARENT, dataset);
 		for (i = 0; i < descriptions_len; ++i)
 			xfree(dataset[i].name);
-		if (debug_flags & DEBUG_FLAG_ENERGY)
-			debug("Energy: dataset created (id = %d)", dataset_id);
+		log_flag(ENERGY, "Energy: dataset created (id = %d)", dataset_id);
 		if (dataset_id == SLURM_ERROR) {
 			error("Energy: Failed to create the dataset for IPMI");
 			return SLURM_ERROR;
@@ -682,7 +679,7 @@ static int _ipmi_send_profile(void)
 			last_time = sensors[id].energy.poll_time;
 	}
 
-	if (debug_flags & DEBUG_FLAG_PROFILE) {
+	if (slurm_conf.debug_flags & DEBUG_FLAG_PROFILE) {
 		for (i = 0; i < descriptions_len; i++) {
 			info("PROFILE-Energy: %sPower=%"PRIu64"",
 			     descriptions[i].label, data[i]);
@@ -703,16 +700,14 @@ static void *_thread_ipmi_run(void *no_data)
 	struct timespec abs;
 
 	flag_energy_accounting_shutdown = false;
-	if (debug_flags & DEBUG_FLAG_ENERGY)
-		info("ipmi-thread: launched");
+	log_flag(ENERGY, "ipmi-thread: launched");
 
 	(void) pthread_setcancelstate(PTHREAD_CANCEL_ENABLE, NULL);
 	(void) pthread_setcanceltype(PTHREAD_CANCEL_ASYNCHRONOUS, NULL);
 
 	slurm_mutex_lock(&ipmi_mutex);
 	if (_thread_init() != SLURM_SUCCESS) {
-		if (debug_flags & DEBUG_FLAG_ENERGY)
-			info("ipmi-thread: aborted");
+		log_flag(ENERGY, "ipmi-thread: aborted");
 		slurm_mutex_unlock(&ipmi_mutex);
 
 		slurm_mutex_lock(&launch_mutex);
@@ -749,8 +744,7 @@ static void *_thread_ipmi_run(void *no_data)
 		slurm_mutex_unlock(&ipmi_mutex);
 	}
 
-	if (debug_flags & DEBUG_FLAG_ENERGY)
-		info("ipmi-thread: ended");
+	log_flag(ENERGY, "ipmi-thread: ended");
 
 	return NULL;
 }
@@ -840,9 +834,18 @@ static int _get_joules_task(uint16_t delta)
 				new->current_watts);
 
 		if (!first) {
-			new->consumed_energy -= start_current_energies[i];
-			new->base_consumed_energy = adjustment +
-				(new->consumed_energy - old->consumed_energy);
+			/* if slurmd is reloaded while the step is alive */
+			if (old->consumed_energy > new->consumed_energy)
+				new->base_consumed_energy =
+					new->consumed_energy + adjustment;
+			else {
+				new->consumed_energy -=
+					start_current_energies[i];
+				new->base_consumed_energy =
+					adjustment +
+					(new->consumed_energy -
+					 old->consumed_energy);
+			}
 		} else {
 			/* This is just for the step, so take all the pervious
 			   consumption out of the mix.
@@ -856,12 +859,9 @@ static int _get_joules_task(uint16_t delta)
 			+ new->base_consumed_energy;
 		memcpy(old, new, sizeof(acct_gather_energy_t));
 
-		if (debug_flags & DEBUG_FLAG_ENERGY)
-			info("_get_joules_task: consumed %"PRIu64" Joules "
-			     "(received %"PRIu64"(%u watts) from slurmd)",
-			     new->consumed_energy,
-			     new->base_consumed_energy,
-			     new->current_watts);
+		log_flag(ENERGY, "%s: consumed %"PRIu64" Joules (received %"PRIu64"(%u watts) from slurmd)",
+			 __func__, new->consumed_energy,
+			 new->base_consumed_energy, new->current_watts);
 	}
 
 	acct_gather_energy_destroy(energies);
@@ -907,7 +907,6 @@ static void _get_node_energy(acct_gather_energy_t *energy)
  */
 extern int init(void)
 {
-	debug_flags = slurm_get_debug_flags();
 	/* put anything that requires the .conf being read in
 	   acct_gather_energy_p_conf_parse
 	*/
@@ -919,7 +918,7 @@ extern int fini(void)
 {
 	uint16_t i;
 
-	if (!running_in_slurmdstepd())
+	if (!running_in_slurmd_stepd())
 		return SLURM_SUCCESS;
 
 	flag_energy_accounting_shutdown = true;
@@ -936,8 +935,10 @@ extern int fini(void)
 	/* clean up the run thread */
 	slurm_cond_signal(&ipmi_cond);
 
-	if (ipmi_ctx)
+	if (ipmi_ctx) {
 		ipmi_monitoring_ctx_destroy(ipmi_ctx);
+		ipmi_ctx = NULL;
+	}
 	reset_slurm_ipmi_conf(&slurm_ipmi_conf);
 
 	slurm_mutex_unlock(&ipmi_mutex);
@@ -945,22 +946,32 @@ extern int fini(void)
 	if (thread_ipmi_id_run)
 		pthread_join(thread_ipmi_id_run, NULL);
 
-	xfree(sensors);
-	xfree(start_current_energies);
+	/*
+	 * We don't really want to destroy the sensors nor the initial state,
+	 * so those values persist a reconfig. And if the process dies, this
+	 * will be lost anyway. So not freeing these variables is not really a
+	 * leak.
+	 *
+	 * xfree(sensors);
+	 * xfree(start_current_energies);
+	 */
 
 	for (i = 0; i < descriptions_len; ++i) {
 		xfree(descriptions[i].label);
 		xfree(descriptions[i].sensor_idxs);
 	}
 	xfree(descriptions);
+	descriptions = NULL;
+	descriptions_len = 0;
 
+	flag_init = false;
 	return SLURM_SUCCESS;
 }
 
 extern int acct_gather_energy_p_update_node_energy(void)
 {
 	int rc = SLURM_SUCCESS;
-	xassert(running_in_slurmdstepd());
+	xassert(running_in_slurmd_stepd());
 
 	return rc;
 }
@@ -974,7 +985,7 @@ extern int acct_gather_energy_p_get_data(enum acct_energy_type data_type,
 	time_t *last_poll = (time_t *)data;
 	uint16_t *sensor_cnt = (uint16_t *)data;
 
-	xassert(running_in_slurmdstepd());
+	xassert(running_in_slurmd_stepd());
 
 	switch (data_type) {
 	case ENERGY_DATA_NODE_ENERGY_UP:
@@ -1036,11 +1047,10 @@ extern int acct_gather_energy_p_set_data(enum acct_energy_type data_type,
 	int rc = SLURM_SUCCESS;
 	int *delta = (int *)data;
 
-	xassert(running_in_slurmdstepd());
+	xassert(running_in_slurmd_stepd());
 
 	switch (data_type) {
 	case ENERGY_DATA_RECONFIG:
-		debug_flags = slurm_get_debug_flags();
 		break;
 	case ENERGY_DATA_PROFILE:
 		slurm_mutex_lock(&ipmi_mutex);
@@ -1319,7 +1329,7 @@ extern void acct_gather_energy_p_conf_set(int context_id_in,
 	}
 
 	context_id = context_id_in;
-	if (!running_in_slurmdstepd())
+	if (!running_in_slurmd_stepd())
 		return;
 
 	if (!flag_init) {
@@ -1330,8 +1340,7 @@ extern void acct_gather_energy_p_conf_set(int context_id_in,
 		if (running_in_slurmd()) {
 			slurm_thread_create(&thread_ipmi_id_launcher,
 					    _thread_launcher, NULL);
-			if (debug_flags & DEBUG_FLAG_ENERGY)
-				info("%s thread launched", plugin_name);
+			log_flag(ENERGY, "%s thread launched", plugin_name);
 		} else
 			_get_joules_task(0);
 	}
